/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2009 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
*******************************************************************************
*
* File      slotcar.c
*
* Author    Milan Brejl
* 
* Version   1.0
* 
* Date      16-Nov-2009
* 
* Brief     Slot car HW related macros and routines
*
*******************************************************************************
* History:
* 1.0 (26-Nov-2009) Initial version
******************************************************************************/
#include "derivative.h" /* include peripheral declarations */
#include "slotcar.h"

/******************************************************************************
* Functions
******************************************************************************/
void SlotCarInit(void) 
{
  /********** Device Initialization **********/
  SOPT1 = 0x00;  // disable COP
  SOPT2 = 0x00;
  SPMSC1 = 0x00; // Disable low-voltage detects
  /* Enable GPIO pullups */
  PTAPE = 0xFF;  // SW on PTA5
  PTBPE = 0x0F;  // EXP(SPI2) on PTB0-3; Don't pull up INT1,2 and Y,Z on PTB4-7
  PTCPE = 0x3F;  // EXP(TX/RX) on PTC3,5; Don't pull up EN/D2 on PTC6
  PTDPE = 0xF7;  // X on PTD0, CON9 on PTD6,7; Don't pull up FB on PTD3
  PTEPE = 0x1F;  // Pull up SD(SPI1) MISO on PTE4
  PTFPE = 0x40;  // Don't pull up LEDs on PTF0-3, IN1,2 on PTF4,5 and D1 on PTF7
  PTGPE = 0x0D;  // CON8 on PTG2,3; Don't pull up SF on PTG1
  /* Initialize GPIO data directions, LED and H-bridge outputs */
  PTADD = 0x00; // all inputs
  PTBDD = 0x00; // all inputs
  PTCD  = 0x00; // outputs low
  PTCDD = 0x40; // EN/D2 output on PTC6
  PTDDD = 0x00; // all inputs
  PTEDD = 0x80; // GPIO Slave Select output on PTE7
  PTED  = 0x80; // SS = 1 (deselected)
  PTFD  = 0x00; // outputs low
  PTFDD = 0xBF; // LED outputs on PTF0-3, IN1,2 on PTF4,5 and D1 on PTF7
  PTGDD = 0x00; // all inputs
  /* Init clock to 48MHz using 8MHz crystal */
  MCGC2 = 0x36;
  while(!(MCGSC & 0x02)){};         // wait for the OSC stable
  MCGC1 = 0x98;
  while((MCGSC & 0x1C) != 0x08){};  // external clock is selected
  MCGC3 = 0x4C;
  while ((MCGSC & 0x48) != 0x48){}; // wait for the PLL is locked
  MCGC1 = 0x18;
  while((MCGSC & 0x6C) != 0x6C){};

  /* initialize RTC for 10ms periodic interrupt */
  RTCMOD = 80;      /* Set RTC modulo = 80 => f = 8MHz/80/1000 */
  RTCSC_RTCLKS = 1; /* Select external clock ERCLK as RTC clock source */
  RTCSC_RTIE = 1;   /* Enable RTC interrupts */
  RTCSC_RTCPS = 8;  /* Select prescaler 1000 and start RTC */
    
  /* initialize TPM2 for motor PWM generation */
  TPM2SC = 0;
  TPM2MOD = 6000;     /* Modulo = 6000 => f = 24MHz/1/(6000*2) = 2kHz */
  TPM2SC_CLKSx = 1;   /* clock source = bus clock */
  TPM2SC_PS = 0;      /* prescaler divider = 1 */
  TPM2SC_CPWMS = 1;   /* center-aligned PWM */
  TPM2C1SC_ELS1x = 1; /* low-true pulses */
  TPM2SC_TOIE = 1;    /* enable overflow interrupt */

  /* initialize the ADC */
  ADCCFG_ADICLK = 1;/* input clock = bus clock divided by 2 */
  ADCCFG_ADIV = 1;  /* input clock div = 2 */
  ADCCFG_ADLSMP = 1;/* long sample time enabled */
  ADCCFG_MODE = 1;  /* 12-bit mode */
  ADCSC2_ADTRG = 0; /* software trigger */
  APCTL1_ADPC6 = 1; /* pin ADP6 is ADC input - accelerometer Z */
  APCTL1_ADPC7 = 1; /* pin ADP7 is ADC input - accelerometer Y */
  APCTL2_ADPC8 = 1; /* pin ADP8 is ADC input - accelerometer X */
  APCTL2_ADPC10 = 1;/* pin ADP10 is ADC input - motor current */
  
  /* initialize Keyboard Interrupt to detect power outage */
  KBI1ES_KBEDG4 = 0; /* detect falling edge from INT1 */
  KBI1ES_KBEDG5 = 0; /* detect falling edge from INT2 */
  KBI1SC_KBIMOD = 1; /* detect edge and level */
  KBI1PE_KBIPE4 = 1; /* enable INT1 pin interrupt */
  KBI1PE_KBIPE5 = 1; /* enable INT2 pin interrupt */
  KBI1SC_KBIE = 1;   /* enable interrupts */
  
  /************ Board HW Initialization *************/

  /* initialize the Motor H-bridge driver */
  PTFD_PTFD7 = 0;  /* set D1 low */
  PTFD_PTFD5 = 0;  /* set IN1 low */
  PTFD_PTFD4 = 0;  /* set IN2 low */
}

/*****************************************************************************/                                                                        
/* Freescale  is  not  obligated  to  provide  any  support, upgrades or new */
/* releases  of  the Software. Freescale may make changes to the Software at */
/* any time, without any obligation to notify or provide updated versions of */
/* the  Software  to you. Freescale expressly disclaims any warranty for the */
/* Software.  The  Software is provided as is, without warranty of any kind, */
/* either  express  or  implied,  including, without limitation, the implied */
/* warranties  of  merchantability,  fitness  for  a  particular purpose, or */
/* non-infringement.  You  assume  the entire risk arising out of the use or */
/* performance of the Software, or any systems you design using the software */
/* (if  any).  Nothing  may  be construed as a warranty or representation by */
/* Freescale  that  the  Software  or  any derivative work developed with or */
/* incorporating  the  Software  will  be  free  from  infringement  of  the */
/* intellectual property rights of third parties. In no event will Freescale */
/* be  liable,  whether in contract, tort, or otherwise, for any incidental, */
/* special,  indirect, consequential or punitive damages, including, but not */
/* limited  to,  damages  for  any loss of use, loss of time, inconvenience, */
/* commercial loss, or lost profits, savings, or revenues to the full extent */
/* such  may be disclaimed by law. The Software is not fault tolerant and is */
/* not  designed,  manufactured  or  intended by Freescale for incorporation */
/* into  products intended for use or resale in on-line control equipment in */
/* hazardous, dangerous to life or potentially life-threatening environments */
/* requiring  fail-safe  performance,  such  as  in the operation of nuclear */
/* facilities,  aircraft  navigation  or  communication systems, air traffic */
/* control,  direct  life  support machines or weapons systems, in which the */
/* failure  of  products  could  lead  directly to death, personal injury or */
/* severe  physical  or  environmental  damage  (High  Risk Activities). You */
/* specifically  represent and warrant that you will not use the Software or */
/* any  derivative  work of the Software for High Risk Activities.           */
/* Freescale  and the Freescale logos are registered trademarks of Freescale */
/* Semiconductor Inc.                                                        */
/*****************************************************************************/